package com.os.ems.info.controller;

import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import com.os.common.annotation.Log;
import com.os.common.core.controller.BaseController;
import com.os.common.core.domain.AjaxResult;
import com.os.common.enums.BusinessType;
import com.os.ems.info.domain.SparePartsInventory;
import com.os.ems.info.domain.SparePartsInventoryExportVO;
import com.os.ems.info.service.ISparePartsInventoryService;
import com.os.common.utils.poi.ExcelUtil;
import com.os.common.core.page.TableDataInfo;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

/**
 * 备件库记录主Controller
 * 
 * @author Yinq
 * @date 2024-12-13
 */
@RestController
@RequestMapping("/ems/info/sparePartsInventory")
public class SparePartsInventoryController extends BaseController
{
    @Autowired
    private ISparePartsInventoryService sparePartsInventoryService;

    /**
     * 查询备件库记录主列表
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:list')")
    @GetMapping("/list")
    public TableDataInfo list(SparePartsInventory sparePartsInventory)
    {
        startPage();
        List<SparePartsInventory> list = sparePartsInventoryService.selectSparePartsInventoryList(sparePartsInventory);
        return getDataTable(list);
    }

    /**
     * 导出备件库记录主列表（支持动态列）
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:export')")
    @Log(title = "备件库记录主", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SparePartsInventory sparePartsInventory) throws IOException
    {
        // 获取包含动态列的数据
        List<SparePartsInventoryExportVO> list = sparePartsInventoryService.selectSparePartsInventoryExportList(sparePartsInventory);
        
        // 收集所有的盘点列名
        Set<String> allCheckNames = new HashSet<>();
        for (SparePartsInventoryExportVO vo : list) {
            if (vo.getCheckData() != null) {
                allCheckNames.addAll(vo.getCheckData().keySet());
            }
        }
        
        // 创建Excel工作簿
        Workbook workbook = new XSSFWorkbook();
        Sheet sheet = workbook.createSheet("备件库记录");
        
        // 创建表头
        Row headerRow = sheet.createRow(0);
        int colIndex = 0;
        
        // 基础列
        String[] baseHeaders = {"主键", "入库时间", "采购方式", "库存位置", "物品名称", "原厂编号", "型号", "入库数量", "剩余数量", "库存阈值", "备注"};
        for (String header : baseHeaders) {
            headerRow.createCell(colIndex++).setCellValue(header);
        }
        
        // 动态盘点列
        List<String> sortedCheckNames = new ArrayList<>(allCheckNames);
        sortedCheckNames.sort(String::compareTo); // 按字母排序
        for (String checkName : sortedCheckNames) {
            headerRow.createCell(colIndex++).setCellValue(checkName);
        }
        
        // 填充数据
        int rowIndex = 1;
        for (SparePartsInventoryExportVO vo : list) {
            Row row = sheet.createRow(rowIndex++);
            colIndex = 0;
            
            // 基础数据
            if (vo.getObjid() != null) row.createCell(colIndex).setCellValue(vo.getObjid());
            colIndex++;
            if (vo.getWarehouseDate() != null) row.createCell(colIndex).setCellValue(vo.getWarehouseDate());
            colIndex++;
            if (vo.getPurchaseMethod() != null) row.createCell(colIndex).setCellValue(vo.getPurchaseMethod());
            colIndex++;
            if (vo.getStorageLocation() != null) row.createCell(colIndex).setCellValue(vo.getStorageLocation());
            colIndex++;
            if (vo.getItemName() != null) row.createCell(colIndex).setCellValue(vo.getItemName());
            colIndex++;
            if (vo.getOriginalPartNumber() != null) row.createCell(colIndex).setCellValue(vo.getOriginalPartNumber());
            colIndex++;
            if (vo.getModel() != null) row.createCell(colIndex).setCellValue(vo.getModel());
            colIndex++;
            if (vo.getWarehouseQuantity() != null) row.createCell(colIndex).setCellValue(vo.getWarehouseQuantity());
            colIndex++;
            if (vo.getRemainingQuantity() != null) row.createCell(colIndex).setCellValue(vo.getRemainingQuantity());
            colIndex++;
            if (vo.getThreshold() != null) row.createCell(colIndex).setCellValue(vo.getThreshold());
            colIndex++;
            if (vo.getRemarks() != null) row.createCell(colIndex).setCellValue(vo.getRemarks());
            colIndex++;
            
            // 动态盘点数据
            for (String checkName : sortedCheckNames) {
                Long value = vo.getCheckValue(checkName);
                if (value != null) {
                    row.createCell(colIndex).setCellValue(value);
                }
                colIndex++;
            }
        }
        
        // 设置响应头
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setHeader("Content-Disposition", "attachment; filename=sparePartsInventory.xlsx");
        
        // 写入响应
        workbook.write(response.getOutputStream());
        workbook.close();
    }

    /**
     * 查询备件库记录主列表（包含动态列）
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:list')")
    @GetMapping("/listWithDynamicColumns")
    public TableDataInfo listWithDynamicColumns(SparePartsInventory sparePartsInventory)
    {
        startPage();
        List<SparePartsInventoryExportVO> list = sparePartsInventoryService.selectSparePartsInventoryExportList(sparePartsInventory);
        return getDataTable(list);
    }

    /**
     * 获取备件库记录主详细信息
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:query')")
    @GetMapping(value = "/{objid}")
    public AjaxResult getInfo(@PathVariable("objid") Long objid)
    {
        return success(sparePartsInventoryService.selectSparePartsInventoryByObjid(objid));
    }

    /**
     * 获取备件库记录主详细信息（包含动态列）
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:query')")
    @GetMapping(value = "/withDynamicColumns/{objid}")
    public AjaxResult getInfoWithDynamicColumns(@PathVariable("objid") Long objid)
    {
        SparePartsInventoryExportVO vo = sparePartsInventoryService.selectSparePartsInventoryWithChecksByObjid(objid);
        return AjaxResult.success(vo);
    }

    /**
     * 新增备件库记录主
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:add')")
    @Log(title = "备件库记录主", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SparePartsInventory sparePartsInventory)
    {
        sparePartsInventory.setCreateBy(getUsername());
        return toAjax(sparePartsInventoryService.insertSparePartsInventory(sparePartsInventory));
    }

    /**
     * 新增备件库记录主（包含动态列）
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:add')")
    @Log(title = "备件库记录主", businessType = BusinessType.INSERT)
    @PostMapping("/withDynamicColumns")
    public AjaxResult addWithDynamicColumns(@RequestBody SparePartsInventoryExportVO sparePartsInventoryVO)
    {
        return toAjax(sparePartsInventoryService.insertSparePartsInventoryWithDynamicColumns(sparePartsInventoryVO));
    }

    /**
     * 修改备件库记录主
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:edit')")
    @Log(title = "备件库记录主", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SparePartsInventory sparePartsInventory)
    {
        sparePartsInventory.setUpdateBy(getUsername());
        return toAjax(sparePartsInventoryService.updateSparePartsInventory(sparePartsInventory));
    }

    /**
     * 修改备件库记录主（包含动态列）
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:edit')")
    @Log(title = "备件库记录主", businessType = BusinessType.UPDATE)
    @PutMapping("/withDynamicColumns")
    public AjaxResult editWithDynamicColumns(@RequestBody SparePartsInventoryExportVO sparePartsInventoryVO)
    {
        return toAjax(sparePartsInventoryService.updateSparePartsInventoryWithDynamicColumns(sparePartsInventoryVO));
    }

    /**
     * 获取所有动态列名
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:list')")
    @GetMapping("/dynamicColumns")
    public AjaxResult getDynamicColumns()
    {
        List<String> columns = sparePartsInventoryService.getAllCheckNames();
        return AjaxResult.success(columns);
    }

    /**
     * 删除备件库记录主
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:remove')")
    @Log(title = "备件库记录主", businessType = BusinessType.DELETE)
	@DeleteMapping("/{objids}")
    public AjaxResult remove(@PathVariable Long[] objids)
    {
        return toAjax(sparePartsInventoryService.deleteSparePartsInventoryByObjids(objids));
    }

    /**
     * 导入备件库记录主数据（支持动态列）
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:add')")
    @Log(title = "备件库记录主", businessType = BusinessType.IMPORT)
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file, @RequestParam(value = "updateSupport", defaultValue = "false") boolean updateSupport) throws Exception
    {
        // 解析Excel文件，支持动态列
        List<SparePartsInventoryExportVO> dataList = parseExcelWithDynamicColumns(file);
        String message = sparePartsInventoryService.importSparePartsInventoryWithDynamicColumns(dataList, updateSupport, getUsername());
        return AjaxResult.success(message);
    }

    /**
     * 解析包含动态列的Excel文件
     */
    private List<SparePartsInventoryExportVO> parseExcelWithDynamicColumns(MultipartFile file) throws Exception {
        List<SparePartsInventoryExportVO> dataList = new ArrayList<>();
        
        Workbook workbook = new XSSFWorkbook(file.getInputStream());
        Sheet sheet = workbook.getSheetAt(0);
        
        // 解析表头
        Row headerRow = sheet.getRow(0);
        Map<Integer, String> columnMap = new LinkedHashMap<>();
        String[] baseColumns = {"主键", "入库时间", "采购方式", "库存位置", "物品名称", "原厂编号", "型号", "入库数量", "剩余数量", "备注"};
        
        for (int i = 0; i < headerRow.getLastCellNum(); i++) {
            Cell cell = headerRow.getCell(i);
            if (cell != null) {
                columnMap.put(i, cell.getStringCellValue());
            }
        }
        
        // 解析数据行
        for (int rowIndex = 1; rowIndex <= sheet.getLastRowNum(); rowIndex++) {
            Row row = sheet.getRow(rowIndex);
            if (row == null) continue;
            
            SparePartsInventoryExportVO vo = new SparePartsInventoryExportVO();
            
            for (Map.Entry<Integer, String> entry : columnMap.entrySet()) {
                int colIndex = entry.getKey();
                String columnName = entry.getValue();
                Cell cell = row.getCell(colIndex);
                
                if (cell == null) continue;
                
                // 处理基础列
                switch (columnName) {
                    case "主键":
                        if (cell.getCellType() == CellType.NUMERIC) {
                            vo.setObjid((long) cell.getNumericCellValue());
                        }
                        break;
                    case "入库时间":
                        vo.setWarehouseDate(getCellStringValue(cell));
                        break;
                    case "采购方式":
                        vo.setPurchaseMethod(getCellStringValue(cell));
                        break;
                    case "库存位置":
                        vo.setStorageLocation(getCellStringValue(cell));
                        break;
                    case "物品名称":
                        vo.setItemName(getCellStringValue(cell));
                        break;
                    case "原厂编号":
                        vo.setOriginalPartNumber(getCellStringValue(cell));
                        break;
                    case "型号":
                        vo.setModel(getCellStringValue(cell));
                        break;
                    case "入库数量":
                        if (cell.getCellType() == CellType.NUMERIC) {
                            vo.setWarehouseQuantity((long) cell.getNumericCellValue());
                        }
                        break;
                    case "剩余数量":
                        if (cell.getCellType() == CellType.NUMERIC) {
                            vo.setRemainingQuantity((long) cell.getNumericCellValue());
                        }
                        break;
                    case "库存阈值":
                        if (cell.getCellType() == CellType.NUMERIC) {
                            vo.setThreshold((long) cell.getNumericCellValue());
                        }
                        break;
                    case "备注":
                        vo.setRemarks(getCellStringValue(cell));
                        break;
                    default:
                        // 处理动态盘点列
                        if (cell.getCellType() == CellType.NUMERIC) {
                            vo.setCheckValue(columnName, (long) cell.getNumericCellValue());
                        }
                        break;
                }
            }
            
            dataList.add(vo);
        }
        
        workbook.close();
        return dataList;
    }

    /**
     * 获取单元格字符串值
     */
    private String getCellStringValue(Cell cell) {
        if (cell == null) return null;
        
        switch (cell.getCellType()) {
            case STRING:
                return cell.getStringCellValue();
            case NUMERIC:
                return String.valueOf((long) cell.getNumericCellValue());
            case BOOLEAN:
                return String.valueOf(cell.getBooleanCellValue());
            default:
                return null;
        }
    }

    /**
     * 获取备件库记录主导入模板
     */
    @PreAuthorize("@ss.hasPermi('ems/info:sparePartsInventory:add')")
    @PostMapping("/importTemplate")
    public AjaxResult importTemplate()
    {
        ExcelUtil<SparePartsInventory> util = new ExcelUtil<SparePartsInventory>(SparePartsInventory.class);
        return util.importTemplateExcel("备件库记录主数据");
    }
}
